<?php
Namespace App\Model\Admin;

Use App\Utils\Utils;
use Symfony\Component\Translation\TranslatorInterface;

/**
 * Provides admin functions for Homegear Influxdb management
 *
 */
class HomegearSystem
{
    public $hg;
    private $translator;
    private $utils;
    private $interfaces = '/etc/network/interfaces';
    private $resolvconf = '/etc/resolvconf/resolv.conf.d/head';

    public function __construct(TranslatorInterface $translator, Utils $utils)
    {
        $this->hg = new \Homegear\Homegear;
        $this->utils = $utils;
        $this->translator = $translator;
    }

    /**
     * Get current network settings
     * 
     * @return array()
     */
    public function getNetworkSettings()
    {
        // check if needed packages are installed
        if ($this->utils->checkManagementRunning(false)) {
            try {
                if (!$this->hg->managementDpkgPackageInstalled("ifupdown")) {
                    $msg[] = $this->translator->trans('system.network.package_ifupdown_missing',array(),'admin');    
                }
            } catch(\Homegear\HomegearException $e) {
                $msg[] = $this->translator->trans('system.network.package_ifupdown_missing',array(),'admin');
            }
            try {
                if (!$this->hg->managementDpkgPackageInstalled("resolvconf")) {
                    $msg[] = $this->translator->trans('system.network.package_resolvconf_missing',array(),'admin');    
                }
            } catch(\Homegear\HomegearException $e) {
                $msg[] = $this->translator->trans('system.network.package_resolvconf_missing',array(),'admin');
            }
            if (isset($msg) && is_array($msg) && count($msg) > 0) {
                // we have at least one error
                return ['result' => 'error', 'message' => $msg];
            }
            // check if files exist
            if (!file_exists($this->interfaces)) {
                $msg[] = $this->translator->trans('system.network.interfaces_missing',array(),'admin');
            } else {
                if(strpos(file_get_contents($this->interfaces),'#{{{ homegear-management') === false) {
                    $msg[] = $this->translator->trans('system.network.interfaces_tag_missing',array(),'admin');
                }
            }
            if (!file_exists($this->resolvconf)) {
                $msg[] = $this->translator->trans('system.network.resolvconf_missing',array(),'admin');
            } else {
                if(strpos(file_get_contents($this->resolvconf),'#{{{ homegear-management') === false) {
                    $msg[] = $this->translator->trans('system.network.resolvconf_tag_missing',array(),'admin');
                }
            }
            if (isset($msg) && is_array($msg) && count($msg) > 0) {
                // we have at least one error
                return ['result' => 'error', 'message' => $msg];
            }
            try {
                $result = $this->hg->managementGetNetworkConfiguration();
                return $result;
            } catch(\Homegear\HomegearException $e) {
                return ['result' => 'error', 'message' => $e->getMessage()];
            }
        }
        return ['result' => 'error', 'message' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    public function setNetworkSettings($params = null)
    {
        if ($params === null) { return; }
        foreach ($params as $key => $value) {
            if ($key != 'dns') {
                // get current IP
                $current = $this->utils->getIpAddresses($key);
                if ($value['ipv4']['type'] == 'dhcp') {
                    $value['ipv4'] = ['type' => $value['ipv4']['type']];
                } else {
                    if (array_key_exists('lastOctet',$value['ipv4'])) {
                        $value['ipv4']['address'] = $value['ipv4']['net'].'.'.$value['ipv4']['lastOctet'];
                        foreach (['net', 'lastOctet'] as $unset) {
                            unset($value['ipv4'][$unset]);
                        }
                    }
                    if ($current['ip'] != $value['ipv4']['address'] && $this->_checkNewIpAddress('4',$value['ipv4']['address'])) {
                        // new ip is not the current one, but it's alive ... so don't set it!
                        $msg = $this->translator->trans('system.network.ip4alive',array(),'admin');
                        return ['status' => 0, 'msg' => $msg];
                    }
                }
                if (array_key_exists('ipv4', $value)) {
                    $params[$key] = $value;
                }
                if ($value['ipv6']['active'] == 'false') {
                    unset($params[$key]['ipv6']);
                } else {
                    if ($value['ipv6']['type'] == 'auto') {
                        $value['ipv6'] = ['type' => $value['ipv6']['type']];
                    } else {
                        if ($current['ipv6'] != $value['ipv6']['address'] && $this->_checkNewIpAddress('6',$value['ipv6']['address'])) {
                            // new ip is not the current one, but it's alive ... so don't set it!
                            unset($value['ipv6']);
                            $msg .= '<br>'.$this->translator->trans('system.network.ip6alive',array(),'admin');
                        }
                    }
                }
            }
        }
        try {
            $this->hg->managementSetNetworkConfiguration($params);
        } catch(\Homegear\HomegearException $e) {
            $msg = $this->translator->trans('system.network.unknownError',array(),'admin');
        }
        if (isset($msg)) {
            return ['status' => 0, 'msg' => $msg];
        }
        return ['status' => 1];
    }

    /**
     * Get current settings of homegear-influxdb
     * 
     * @return array()
     */
    public function getInfluxdbSettings()
    {
        if ($this->utils->checkManagementRunning(false)) {
            try {
                foreach (['enabled', 'hostname', 'port', 'username', 'password', 'databaseName'] as $setting) {
                    $result[$setting] = $this->hg->managementGetConfigurationEntry('influxdb.conf',$setting);
                }
                return $result;
            } catch(\Homegear\HomegearException $e) {
                return ['result' => 'error', 'message' => $e->getMessage()];
            }
        }
        return ['result' => 'error', 'msg' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    /**
     * Update settings of homegear-influxdb
     * 
     * @return array()
     */
    public function setInfluxdbSettings($params = null)
    {
        if ($params === null || !is_array($params)) { return false; }
        if ($this->utils->checkManagementRunning(false)) {
            foreach (['enabled', 'hostname', 'port', 'username', 'password', 'databaseName'] as $setting) {
                if ($setting == 'enabled' && !array_key_exists($setting,$params)) { continue; }
                if ($setting == 'username' && $params[$setting] == '') { continue; }
                if ($setting == 'password' && $params[$setting] == '') { continue; }
                $this->hg->managementSetConfigurationEntry('influxdb.conf',$setting, $params[$setting]);
            }
            return ['result' => 'success'];
        }
        return ['result' => 'error', 'msg' => $this->translator->trans('influxdb.settings.unable_to_update_settings',array(),'admin')];
    }

    /**
     * Get current settings of homegear-mqtt
     * 
     * @return array()
     */
    public function getMqttSettings()
    {
        if ($this->utils->checkManagementRunning(false)) {
            try {
                foreach (['enabled', 'brokerHostname', 'brokerPort', 'clientName', 'prefix', 'homegearId', 'retain', 'username', 'password', 'plainTopic', 'jsonTopic', 'jsonobjTopic', 'enableSSL'] as $setting) {
                    $result[$setting] = $this->hg->managementGetConfigurationEntry('mqtt.conf',$setting);
                }
                return $result;
            } catch(\Homegear\HomegearException $e) {
                return ['result' => 'error', 'message' => $e->getMessage()];
            }
        }
        return ['result' => 'error', 'msg' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    /**
     * Update settings of homegear-mqtt
     * 
     * @return array()
     */
    public function setMqttSettings($params = null)
    {
        if ($params === null || !is_array($params)) { return false; }
        if ($this->utils->checkManagementRunning(false)) {
            foreach (['enabled', 'brokerHostname', 'brokerPort', 'clientName', 'prefix', 'homegearId', 'retain', 'username', 'password', 'plainTopic', 'jsonTopic', 'jsonobjTopic', 'enableSSL'] as $setting) {
                if ($setting == 'enabled' && $params[$setting] == '') { continue; }
                if ($setting == 'username' && $params[$setting] == '') { continue; }
                if ($setting == 'password' && $params[$setting] == '') { continue; }
                if ($setting == 'enableSSL' && !array_key_exists($setting,$params)) { continue; }
                if ($setting == 'clientName' && !array_key_exists($setting,$params)) { 
                    $params['clientName'] = $params['homegearId'];
                }
                $this->hg->managementSetConfigurationEntry('mqtt.conf',$setting, $params[$setting]);
            }
            return ['result' => 'success'];
        }
        return ['result' => 'error', 'msg' => $this->translator->trans('mqtt.settings.unable_to_update_settings',array(),'admin')];
    }

    /**
     * Get tts settings for sonos
     * 
     * @return array()
     */
    public function getTtsSettings()
    {
        foreach (['ttsProgram', 'ttskey', 'ttsusername'] as $setting) {
            if ($setting == 'ttsProgram') {
                if ($this->utils->checkManagementRunning(false)) {
                    try {
                        $ttsSettings[$setting] = $this->hg->managementGetConfigurationEntry('families/sonos.conf',$setting);
                        $ttsSettings[$setting] = preg_replace('/homegear -e rs DeviceScripts\/Sonos\//','',$ttsSettings[$setting]);
                    } catch(\Homegear\HomegearException $e) {
                        return ['result' => 'error', 'message' => $e->getMessage()];
                    }
                }
            } else {
                try {
                    $ttsSettings[$setting] = $this->hg->getFamilySetting(6, $setting);
                } catch(\Homegear\HomegearException $e) {
                    $ttsSettings[$setting] = '';
                }
            }
        }
        return $ttsSettings;
    }

    public function setTtsSettings($params = null)
    {
        if ($params === null || !is_array($params)) { return false; }
        foreach (['ttsProgram', 'ttskey', 'ttsusername'] as $setting) {
            if ($setting == 'ttsProgram') {
                if ($this->utils->checkManagementRunning(false)) {
                    try {
                        $this->hg->managementSetConfigurationEntry('families/sonos.conf',$setting, 'homegear -e rs DeviceScripts/Sonos/'.$params[$setting]);
                    } catch(\Homegear\HomegearException $e) {
                        return ['result' => 'error', 'msg' => $this->translator->trans('tts.settings.unable_to_update_settings',array(),'admin')];
                    }
                }
            } else {
                try {
                    $this->hg->setFamilySetting(6, $setting, $params[$setting]);
                } catch(\Homegear\HomegearException $e) {
                    return ['result' => 'error', 'msg' => $this->translator->trans('tts.settings.unable_to_update_settings',array(),'admin')];
                }
            }
        }
        return ['result' => 'success'];
    }

    /**
     * Restart a service
     * 
     * @return array()
     */
    public function restartService($service = null)
    {
        if ($service === null) { return; }
        if ($this->utils->checkManagementRunning(false)) {
            // stop homegear-management
            $stopId = $this->hg->managementServiceCommand("$service",'stop');
            $stopResult = $this->utils->getCommandStatus($stopId);
            if ($stopResult['exitCode'] == 0) {
                // stop was successful, start again
                $startId = $this->hg->managementServiceCommand("$service",'start');
                $startResult = $this->utils->getCommandStatus($startId);
                if ($startResult['exitCode'] == 0) {
                    return ['result' => 'success', 'msg' => $this->translator->trans('influxdb.settings.update_done',array(),'admin')];
                }
                return ['result' => 'error', 'service' => $service, 'msg' => $this->translator->trans('influxdb.settings.unable_to_start_service',array(),'admin')];
            }
            return ['result' => 'error', 'service' => $service, 'msg' => $this->translator->trans('influxdb.settings.unable_to_stop_service',array(),'admin')];
        }
        return ['result' => 'error', 'msg' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    /**
     * System reset
     * 
     */
    public function doReset()
    {
        try {
            $this->hg->managementSystemReset();
        } catch(\Homegear\HomegearException $e) {
        }
    }

    private function _checkNewIpAddress($type = null, $address = null)
    {
        if ($type === null || $address === null) { return false; }
        exec(sprintf('ping -'.$type.' -c 2 -W 5 %s', escapeshellarg($address)), $res, $rval);
        if(stristr(implode($res), '100% packet loss') === FALSE) {
            return true;
        }
        return false;
    }
}
