<?php
Namespace App\Model;

/**
 * Provides base functions for Homegear stories
 *
 */
class HomegearStory
{
    const componentStorySort = 'sortStories';
    const componentStoryGridSort = 'sortStoryGrid';
    public $hg;
    public $defaultLanguage;
    public $params;

    public function __construct($params)
    {
        $this->hg = new \Homegear\Homegear;
        $this->defaultLanguage = $params->get('default_locale');
        $this->params = $params;
    }

    /**
     * Load all known stories
     * 
     * @return array()
     */
    public function loadAll($withBuildings = false)
    {
        // load all rooms, return if there are none
        try {
            $stories = $this->hg->getStories();
        } catch(\Homegear\HomegearException $e) {
            return;
        }
        if (is_array($stories) && count($stories) == 0) { return; }
        // DEPRECATED: sorting is done by homegear directly!
        $stories = $this->_sortStories($stories);
        if ($withBuildings === true) {
            $stories = $this->_getBuildingsForStories($stories);
        }
        return $stories;
    }

    /**
     * Load a specific story w/ rooms
     * 
     * @return array()
     */
    public function loadById($id = null)
    {
        $allStories = $this->loadAll();
        if ($id === null || $id == '') { 
            // no id given, load first room
            if (is_array($allStories) && count($allStories) > 0) {
                $id = $allStories[0]['ID'];
            }
        }
        if ($id !== null) {
            // get the story metadata out of the story-array
            $result = $allStories[array_search($id, array_column($allStories,'ID'))];
            // load sorting
            $sort = json_decode($this->hg->getData($this::componentStoryGridSort,"$id"),true);
            // load rooms
            foreach ($result['ROOMS'] as $k => $v) {
                $key = 0;
                if (is_array($sort) && count($sort) > 0) {
                    $key = array_search($v, $sort);
                }
                $result['ROOMS'][$k] = ['sort' => (isset($key) ? $key : 0), 'id' => $v];
            }
            $result['ROOMS'] = ((is_array($result['ROOMS']) && count($result['ROOMS']) > 0) ? array_values($result['ROOMS']) : null);
            return $result;
        }
    }

    public function loadForBuilding($building = false)
    {
        if ($building === false) { return; }
        if (array_key_exists('STORIES',$building) && is_array($building['STORIES']) && count($building['STORIES']) > 0) {
            $allStories = $this->loadAll();
            usort($building['STORIES'], function($a, $b) { return $a['sort'] <=> $b['sort']; });
            foreach ($building['STORIES'] as $key => $story) {
                $currentStory = array_search($story['id'], array_column($allStories,'ID'));
                $currentStories[] = $allStories[$currentStory];
            }
            return $currentStories;
        }
        return [];
    }

    private function _getBuildingsForStories($stories)
    {
        $building = new HomegearBuilding($this->params);
        $buildings = $building->loadAll(true);
        if (is_array($buildings)) {
            foreach ($buildings as $b) {
                if (array_key_exists('STORIES',$b)) {
                    foreach ($b['STORIES'] as $key => $bstory) {
                        $key = array_search($bstory, array_column($stories, 'ID'));
                        if (isset($key) && $key !== NULL) {
                            $stories[$key]['BUILDING'] = [ 'ID' => $b['ID'], 'NAME' => $b['TRANSLATIONS']];
                        }
                    }
                }
            }
        }
        return $stories;
    }

    /**
     * Sort stories
     * 
     * DEPRECATED: sorting is done by homegear directly!
     * This is only needed once to convert from db-style to homegear-style
     * @return array()
     */
    private function _sortStories($stories)
    {
        // load sorting data
        $sort = $this->hg->getData($this::componentStorySort);
        if ($sort !== null) {
            foreach ($stories as $story) {
                if (array_key_exists($story['ID'],$sort)) {
                    $metadata = $this->hg->getStoryMetadata($story['ID']);
                    if(!is_array($metadata)) $metadata = array();
                    $metadata['position'] = intval($sort[$story['ID']]);
                    $this->hg->setStoryMetadata($story['ID'], $metadata);
                }
            }
            $stories = $this->sortDataByComponent($stories,$sort);
            // delete sorting data since it's not needed anymore
            $this->hg->deleteData($this::componentStorySort);
        }
        return $stories;
    }

    /**
     * DEPRECATED: sorting is done by homegear directly!
     * 
     */
    private function sortDataByComponent($dataToSort,$component)
    {
        foreach ($dataToSort as $k => $v) {
            $dataToSort[$k]['sort'] = (isset($component[$v['ID']]) ? $component[$v['ID']] : 0);
        }
        usort($dataToSort, function($a, $b) {
            $retval = $a['sort'] <=> $b['sort'];
            if ($retval == 0) { $retval = $a['ID'] <=> $b['ID']; }
            return $retval;
        });
        return $dataToSort;
    }

}